@@ -17,22 +17,25 @@ if [ ! -z $"HERMIT_EXE" ]; then
   unset HERMIT_EXE
 fi
 
-# Creates a "fake" release directory and sets HERMIT_DIST_URL to refer to it
+# Creates a "fake" release directory and sets HERMIT_DIST_URL to refer to it.
+# This function expects the basename of the first argument to be the
+# channel name, e.g., "release/canary" or "release/stable"
 fakeRelease() {
   DIR=$1
+  CHANNEL="$(basename "${DIR}")"
 
   echo "Compiling hermit"
   (
     . ../../bin/activate-hermit
-    go build -o hermit ../../cmd/hermit
-    go install ../../cmd/geninstaller
+    go build -ldflags "-X main.channel=${CHANNEL}" -o hermit ../../cmd/hermit
   )
 
   OS=$(../../bin/go version | awk '{print $NF}' | cut -d/ -f1)
   ARCH=$(../../bin/go version | awk '{print $NF}' | cut -d/ -f2)
   mkdir -p "$DIR"
   gzip -c hermit > "$DIR/hermit-${OS}-${ARCH}.gz"
-  ../../.hermit/go/bin/geninstaller --dest="${DIR}/install.sh" --dist-url=-https://github.com/cashapp/hermit/releases/download/stable
+  INSTALLER_VERSION=$(./hermit gen-installer --dest="${DIR}/install.sh")
+  cp "${DIR}/install.sh" "${DIR}/install-${INSTALLER_VERSION}.sh"
 
   export HERMIT_DIST_URL=file://$PWD/$DIR
   echo $HERMIT_DIST_URL
