@@ -12,23 +12,26 @@ import (
 	"strings"
 	"time"
 
+	"github.com/alecthomas/participle/v2"
 	"github.com/gobwas/glob"
-
-	"github.com/pkg/errors"
 	"github.com/qdm12/reprint"
 
 	"github.com/cashapp/hermit/envars"
+	"github.com/cashapp/hermit/errors"
+	"github.com/cashapp/hermit/internal/system"
+	"github.com/cashapp/hermit/platform"
 	"github.com/cashapp/hermit/sources"
 	"github.com/cashapp/hermit/ui"
 )
 
 // ErrUnknownPackage is returned when a package cannot be resolved.
 var ErrUnknownPackage = errors.New("unknown package")
-var xarch = map[string]string{
-	"amd64": "x86_64",
-	"386":   "i386",
-	"arm64": "aarch64",
-}
+
+// ErrNoBinaries is returned when a resolved package does not contain binaries or apps
+var ErrNoBinaries = errors.New("no binaries or apps provided")
+
+// ErrNoSource is returned when a resolved package does not contain source
+var ErrNoSource = errors.New("no source provided")
 
 // Config required for loading manifests.
 type Config struct {
@@ -36,10 +39,7 @@ type Config struct {
 	Env string
 	// State path where packages are installed.
 	State string
-	// Optional OS (will use runtime.GOOS if not provided).
-	OS string
-	// Optional Arch (will use runtime.GOARCH if not provided).
-	Arch string
+	platform.Platform
 }
 
 // Packages sortable by name + version.
@@ -67,32 +67,37 @@ type ResolvedFileRef struct {
 
 // Package resolved from a manifest.
 type Package struct {
-	Description    string
-	Reference      Reference
-	Arch           string
-	Binaries       []string
-	Apps           []string
-	Requires       []string
-	Provides       []string
-	Rename         map[string]string `json:"-"`
-	Env            envars.Ops
-	Source         string
-	Mirrors        []string
-	Root           string
-	SHA256         string
-	Dest           string
-	Test           string
-	Strip          int
-	Triggers       map[Event][]Action `json:"-"` // Triggers keyed by event.
-	UpdateInterval time.Duration      // How often should we check for updates? 0, if never
-	Files          []*ResolvedFileRef `json:"-"`
-	FS             fs.FS              `json:"-"` // FS the Package was loaded from.
-	Warnings       []string           `json:"-"`
+	Description          string
+	Homepage             string
+	Repository           string
+	Reference            Reference
+	Arch                 string
+	Binaries             []string
+	Apps                 []string
+	Requires             []string
+	RuntimeDeps          []Reference
+	Provides             []string
+	Env                  envars.Ops
+	Source               string
+	SHA256Source         string
+	DontExtract          bool // Don't extract the package, just download it.
+	Mirrors              []string
+	Root                 string
+	SHA256               string
+	Mutable              bool
+	Dest                 string
+	Test                 string
+	Strip                int
+	Triggers             map[Event][]Action  `json:"-"` // Triggers keyed by event.
+	UpdateInterval       time.Duration       // How often should we check for updates? 0, if never
+	Files                []*ResolvedFileRef  `json:"-"`
+	FS                   fs.FS               `json:"-"` // FS the Package was loaded from.
+	Warnings             []string            `json:"-"`
+	UnsupportedPlatforms []platform.Platform // Unsupported core platforms
 
 	// Filled in by Env.
 	Linked    bool `json:"-"` // Linked into environment.
 	State     PackageState
-	LastUsed  time.Time
 	ETag      string
 	UpdatedAt time.Time
 }
@@ -150,6 +155,19 @@ func (p *Package) DeprecationWarningf(format string, args ...interface{}) {
 	p.Warnings = append(p.Warnings, fmt.Sprintf("DEPRECATED: "+format, args...))
 }
 
+// Unsupported package in this environment.
+func (p *Package) Unsupported() bool {
+	return p.Source == ""
+}
+
+// EnsureSupported returns an error if the package is not supported on this platform
+func (p *Package) EnsureSupported() error {
+	if p.Unsupported() {
+		return errors.Errorf("package %s is not supported on this architecture", p.Reference)
+	}
+	return nil
+}
+
 // Resolver of packages.
 type Resolver struct {
 	config  Config
@@ -184,6 +202,10 @@ func (r *Resolver) Errors() ManifestErrors {
 }
 
 // Sync the sources of this resolver.
+//
+// Will be synced at most every SyncFrequency unless "force" is true.
+//
+// A Sources set can only be synchronised once. Following calls will not have any effect.
 func (r *Resolver) Sync(l *ui.UI, force bool) error {
 	if err := r.sources.Sync(l, force); err != nil {
 		return errors.WithStack(err)
@@ -204,7 +226,13 @@ func (r *Resolver) Search(l ui.Logger, pattern string) (Packages, error) {
 		return nil, errors.WithStack(err)
 	}
 	for _, manifest := range manifests {
-		if !re.MatchString(manifest.Name) {
+		if !re.MatchString(manifest.Name) && !re.MatchString(manifest.Description) && !re.MatchString(manifest.Homepage) {
+			continue
+		}
+		if len(manifest.Errors) > 0 {
+			for _, err := range manifest.Errors {
+				l.Warnf("%s:%s", manifest.Path, err)
+			}
 			continue
 		}
 		for _, version := range manifest.Versions {
@@ -212,11 +240,12 @@ func (r *Resolver) Search(l ui.Logger, pattern string) (Packages, error) {
 				ref := Reference{Name: manifest.Name, Version: ParseVersion(vstr)}
 				// If the reference doesn't resolve, discard it.
 				pkg, err := newPackage(manifest, r.config, ExactSelector(ref))
-				if err != nil {
+				if errors.Is(err, ErrNoSource) || errors.Is(err, ErrNoBinaries) || err == nil {
+					pkgs = append(pkgs, pkg)
+				} else {
 					l.Warnf("invalid manifest reference %s in %s.hcl: %s", ref, manifest.Name, err)
 					continue
 				}
-				pkgs = append(pkgs, pkg)
 			}
 		}
 		for _, channel := range manifest.Channels {
@@ -243,6 +272,9 @@ func (r *Resolver) ResolveVirtual(name string) (pkgs []*Package, err error) {
 	}
 	var providers []*AnnotatedManifest
 	for _, manifest := range manifests {
+		if len(manifest.Errors) > 0 {
+			continue
+		}
 		for _, provides := range manifest.Provides {
 			if provides == name {
 				providers = append(providers, manifest)
@@ -266,18 +298,10 @@ func (r *Resolver) ResolveVirtual(name string) (pkgs []*Package, err error) {
 // Resolve a package reference.
 //
 // Returns the highest version matching the given reference
-func (r *Resolver) Resolve(l *ui.UI, selector Selector) (pkg *Package, err error) {
-	manifest, err := r.loader.Get(selector.Name())
+func (r *Resolver) Resolve(l *ui.UI, selector Selector) (*Package, error) {
+	manifest, err := r.loader.Load(l, selector.Name())
 	if err != nil {
-		err := r.Sync(l, true)
-		if err != nil {
-			return nil, errors.Wrap(err, err.Error())
-		}
-		// Try again.
-		manifest, err = r.loader.Get(selector.Name())
-		if err != nil {
-			return nil, errors.WithStack(err)
-		}
+		return nil, errors.WithStack(err)
 	}
 	return newPackage(manifest, r.config, selector)
 }
@@ -307,13 +331,18 @@ func matchChannel(manifest *AnnotatedManifest, selector Selector) (collected Ref
 	return
 }
 
+// Resolve a concrete [Package] reference for the given Config.
+func Resolve(manifest *AnnotatedManifest, config Config, ref Reference) (*Package, error) {
+	return newPackage(manifest, config, ExactSelector(ref))
+}
+
 func newPackage(manifest *AnnotatedManifest, config Config, selector Selector) (*Package, error) {
 	// If a version was not specified and the manifest defines a default, use it.
 	if !selector.IsFullyQualified() && manifest.Default != "" {
 		if strings.HasPrefix(manifest.Default, "@") {
 			selector = ExactSelector(Reference{Name: manifest.Name, Channel: manifest.Default[1:]})
 		} else {
-			m, err := GlobSelector(manifest.Name + "-" + manifest.Default)
+			m, err := ParseGlobSelector(manifest.Name + "-" + manifest.Default)
 			if err != nil {
 				return nil, errors.WithStack(err)
 			}
@@ -333,31 +362,54 @@ func newPackage(manifest *AnnotatedManifest, config Config, selector Selector) (
 		channelRefs, foundUpdateInterval, found = matchChannel(manifest, selector)
 		allRefs = append(allRefs, channelRefs...)
 	}
+	if len(allRefs) == 0 {
+		return nil, errors.Errorf("could not find any versions matching %s", selector)
+	}
 	// Finally just pick the most recent version.
 	if !found.IsSet() && !selector.IsFullyQualified() {
 		sort.Sort(allRefs)
 		found = allRefs[len(allRefs)-1]
 	}
 	if !found.IsSet() {
-		knownVersions := make([]string, 0, len(allRefs))
+		var knownVersions []string
+		var knownChannels []string
 		for _, ref := range allRefs {
-			knownVersions = append(knownVersions, ref.StringNoName())
+			if ref.IsChannel() {
+				knownChannels = append(knownChannels, ref.String())
+			} else {
+				knownVersions = append(knownVersions, ref.String())
+			}
 		}
 		sort.Strings(knownVersions)
-		return nil, errors.Wrapf(ErrUnknownPackage, "%s: no version %s in known versions %s", manifest.Path, selector, strings.Join(knownVersions, ", "))
+		sort.Strings(knownChannels)
+		if strings.Contains(selector.String(), "@") {
+			tryVersion := strings.ReplaceAll(selector.String(), "@", "-")
+			for _, ver := range knownVersions {
+				if ver == tryVersion {
+					return nil, errors.Wrapf(ErrUnknownPackage, "%s: no channel %s found, did you mean version %s?",
+						manifest.Path, selector, tryVersion)
+				}
+			}
+			return nil, errors.Wrapf(ErrUnknownPackage, "%s: no channel %s found in channels (%s) or versions (%s)",
+				manifest.Path, selector, strings.Join(knownChannels, ", "), strings.Join(knownVersions, ", "))
+		}
+		return nil, errors.Wrapf(ErrUnknownPackage, "%s: no version %s found in versions (%s) or channels (%s)",
+			manifest.Path, selector, strings.Join(knownVersions, ", "), strings.Join(knownChannels, ", "))
 	}
 
 	root := filepath.Join(config.State, "pkg", found.String())
 	p := &Package{
-		Description:    manifest.Description,
-		Reference:      found,
-		Rename:         map[string]string{},
-		Root:           "${dest}",
-		Dest:           root,
-		Triggers:       map[Event][]Action{},
-		UpdateInterval: foundUpdateInterval,
-		Files:          []*ResolvedFileRef{},
-		FS:             manifest.FS,
+		Description:          manifest.Description,
+		Homepage:             manifest.Homepage,
+		Repository:           manifest.Repository,
+		Reference:            found,
+		Root:                 "${dest}",
+		Dest:                 root,
+		Triggers:             map[Event][]Action{},
+		UpdateInterval:       foundUpdateInterval,
+		Files:                []*ResolvedFileRef{},
+		FS:                   manifest.FS,
+		UnsupportedPlatforms: manifest.unsupported(found, platform.Core),
 	}
 
 	files := map[string]string{}
@@ -383,16 +435,20 @@ func newPackage(manifest *AnnotatedManifest, config Config, selector Selector) (
 		}
 	}
 
+	vars := map[string]string{}
 	layerEnvars := make([]envars.Envars, 0, len(layers))
 	for _, layer := range layers {
 		if len(layer.Env) > 0 {
 			layerEnvars = append(layerEnvars, layer.Env)
 		}
+		for k, v := range layer.Vars {
+			vars[k] = v
+		}
 		if layer.Arch != "" {
 			p.Arch = layer.Arch
 		}
-		if layer.SHA256 != "" {
-			p.SHA256 = layer.SHA256
+		if layer.Mutable {
+			p.Mutable = layer.Mutable
 		}
 		if layer.Test != nil {
 			p.Test = *layer.Test
@@ -400,6 +456,12 @@ func newPackage(manifest *AnnotatedManifest, config Config, selector Selector) (
 		if layer.Source != "" {
 			p.Source = layer.Source
 		}
+		if layer.SHA256Source != "" {
+			p.SHA256Source = layer.SHA256Source
+		}
+		if layer.DontExtract {
+			p.DontExtract = layer.DontExtract
+		}
 		if len(layer.Mirrors) > 0 {
 			p.Mirrors = layer.Mirrors
 		}
@@ -421,24 +483,27 @@ func newPackage(manifest *AnnotatedManifest, config Config, selector Selector) (
 		if len(layer.Provides) != 0 {
 			p.Provides = append(p.Provides, layer.Provides...)
 		}
-		for k, v := range layer.Rename {
-			p.Rename[k] = v
-		}
 		if len(layer.Triggers) > 0 {
 			for _, trigger := range layer.Triggers {
 				p.Triggers[trigger.Event] = append(p.Triggers[trigger.Event], trigger.Ordered()...)
 			}
 		}
+		if len(layer.RuntimeDeps) > 0 {
+			for _, dep := range layer.RuntimeDeps {
+				ref := ParseReference(dep)
+				p.RuntimeDeps = append(p.RuntimeDeps, ref)
+			}
+		}
 		for k, v := range layer.Files {
 			files[k] = v
 		}
 	}
-	// Validate.
+	// Verify.
 	if len(p.Binaries) == 0 && len(p.Apps) == 0 {
-		return nil, errors.Errorf("%s: %s: no binaries or apps provided", manifest.Path, found)
+		return p, errors.Wrapf(ErrNoBinaries, "%s: %s", manifest.Path, found)
 	}
 	if p.Source == "" {
-		return nil, errors.Errorf("%s: %s: no source provided", manifest.Path, found)
+		return p, errors.Wrapf(ErrNoSource, "%s: %s", manifest.Path, found)
 	}
 
 	// Expand variables.
@@ -446,13 +511,16 @@ func newPackage(manifest *AnnotatedManifest, config Config, selector Selector) (
 	// If "ignoreMissing" is false, any referenced variables that are unknown will result in an error.
 	//
 	// TODO: Factor this out (there's a lot of captured state though).
-	home, err := os.UserHomeDir()
+	home, err := system.UserHomeDir()
 	if err != nil {
 		return nil, errors.WithStack(err)
 	}
 	mapping := func(ignoreMissing bool) func(s string) string {
 		return func(key string) string {
 			switch key {
+			case "name":
+				return found.Name
+
 			case "version":
 				return found.Version.String()
 
@@ -475,9 +543,8 @@ func newPackage(manifest *AnnotatedManifest, config Config, selector Selector) (
 				return config.Arch
 
 			case "xarch":
-				subst, ok := xarch[config.Arch]
-				if ok {
-					return subst
+				if xarch := platform.ArchToXArch(config.Arch); xarch != "" {
+					return xarch
 				}
 				return config.Arch
 
@@ -494,6 +561,10 @@ func newPackage(manifest *AnnotatedManifest, config Config, selector Selector) (
 				return fmt.Sprintf("%02d", time.Now().Day())
 
 			default:
+				value, ok := vars[key]
+				if ok {
+					return value
+				}
 				if ignoreMissing {
 					return "${" + key + "}"
 				}
@@ -540,17 +611,21 @@ func newPackage(manifest *AnnotatedManifest, config Config, selector Selector) (
 	for i, provides := range p.Provides {
 		p.Provides[i] = expand(provides, false)
 	}
-	if len(p.Rename) > 0 {
-		p.DeprecationWarningf(`rename = {"X": "Y"} must be replaced by on unpack { rename { from="${root}/X" to="${root}/Y" } }`)
-	}
-	for k, v := range p.Rename {
-		delete(p.Rename, k)
-		p.Rename[expand(k, true)] = expand(v, true)
-	}
 	p.Source = expand(p.Source, false)
+	p.SHA256Source = expand(p.SHA256Source, false)
 	for i, mirror := range p.Mirrors {
 		p.Mirrors[i] = expand(mirror, false)
 	}
+	// Get sha256 checksum after variable expansion for source, taking care of
+	// autoversion
+	for _, layer := range layers {
+		if layer.SHA256 != "" {
+			p.SHA256 = layer.SHA256
+		} else if sum, ok := manifest.SHA256Sums[p.Source]; ok {
+			p.SHA256 = sum
+		}
+	}
+	inferPackageRepository(p, manifest.Manifest)
 	for _, actions := range p.Triggers {
 		for _, action := range actions {
 			switch action := action.(type) {
@@ -562,18 +637,44 @@ func newPackage(manifest *AnnotatedManifest, config Config, selector Selector) (
 					action.Args[i] = expand(arg, false)
 				}
 				action.Command = expand(action.Command, false)
+				if err := mustAbs(action, action.Command); err != nil {
+					return nil, err
+				}
 				action.Dir = expand(action.Dir, false)
+				if err := mustAbs(action, action.Dir); err != nil {
+					return nil, err
+				}
 
 			case *CopyAction:
 				action.From = expand(action.From, false)
 				action.To = expand(action.To, false)
+				if err := mustAbs(action, action.To); err != nil {
+					return nil, err
+				}
 
 			case *ChmodAction:
 				action.File = expand(action.File, false)
+				if err := mustAbs(action, action.File); err != nil {
+					return nil, err
+				}
 
 			case *RenameAction:
 				action.From = expand(action.From, false)
+				if err := mustAbs(action, action.From); err != nil {
+					return nil, err
+				}
 				action.To = expand(action.To, false)
+				if err := mustAbs(action, action.To); err != nil {
+					return nil, err
+				}
+
+			case *DeleteAction:
+				for i := range action.Files {
+					action.Files[i] = expand(action.Files[i], false)
+					if err := mustAbs(action, action.Files[i]); err != nil {
+						return nil, err
+					}
+				}
 
 			case *MessageAction:
 				action.Text = expand(action.Text, false)
@@ -598,30 +699,67 @@ func newPackage(manifest *AnnotatedManifest, config Config, selector Selector) (
 	return p, err
 }
 
-func resolveFiles(manifest *AnnotatedManifest, pkg *Package, files map[string]string) error {
-	if len(files) == 0 {
-		return nil
+func inferPackageRepository(p *Package, manifest *Manifest) {
+	// start infer from source if no repository is given
+	if p == nil || p.Repository != "" || p.Source == "" {
+		return
 	}
 
-	for k, v := range files {
-		f, err := manifest.FS.Open(k)
-		if err != nil {
-			return errors.WithStack(err)
+	githubComPrefix := "https://github.com/"
+
+	if manifest != nil {
+		for _, v := range manifest.Versions {
+			if v.AutoVersion != nil && v.AutoVersion.GitHubRelease != "" {
+				p.Repository = fmt.Sprintf("%s%s", githubComPrefix, v.AutoVersion.GitHubRelease)
+				return
+			}
 		}
-		err = f.Close()
-		if err != nil {
-			return errors.WithStack(err)
+	}
+
+	if strings.HasPrefix(p.Source, githubComPrefix) == false || strings.HasPrefix(p.Source, "https://github.com/cashapp/hermit-build") {
+		return
+	}
+
+	rest := strings.TrimPrefix(p.Source, githubComPrefix)
+
+	restSplit := strings.Split(rest, "/")
+
+	if len(restSplit) < 2 { //
+		return
+	}
+
+	result := fmt.Sprintf("%s%s", githubComPrefix, strings.Join(restSplit[0:2], "/"))
+
+	p.Repository = result
+}
+
+// HighestMatch returns the VersionBlock with highest version number matching the given Glob
+func (m *Manifest) HighestMatch(to glob.Glob) (result *VersionBlock, highest *Version) {
+	versions := m.Versions
+	for _, v := range versions {
+		block := v
+		for _, vstr := range v.Version {
+			parsed := ParseVersion(vstr)
+			if to.Match(vstr) && (highest == nil || highest.Less(parsed)) {
+				highest = &parsed
+				result = &block
+			}
+		}
+	}
+	return
+}
+
+// ChannelByName returns the channel with the given name, or nil if not found
+func (m *Manifest) ChannelByName(name string) *ChannelBlock {
+	for _, c := range m.Channels {
+		if c.Name == name {
+			return &c
 		}
-		pkg.Files = append(pkg.Files, &ResolvedFileRef{
-			FromPath: k,
-			FS:       manifest.FS,
-			ToPAth:   v,
-		})
 	}
 	return nil
 }
 
-// Validate that there are no semantic errors in the manifest
+// Verify that there are no semantic errors in the manifest
 func (m *Manifest) validate() []error {
 	var (
 		result   []error
@@ -652,28 +790,33 @@ func (m *Manifest) validate() []error {
 	return result
 }
 
-// HighestMatch returns the VersionBlock with highest version number matching the given Glob
-func (m *Manifest) HighestMatch(to glob.Glob) (result *VersionBlock, highest *Version) {
-	versions := m.Versions
-	for _, v := range versions {
-		block := v
-		for _, vstr := range v.Version {
-			parsed := ParseVersion(vstr)
-			if to.Match(vstr) && (highest == nil || highest.Less(parsed)) {
-				highest = &parsed
-				result = &block
-			}
-		}
+func resolveFiles(manifest *AnnotatedManifest, pkg *Package, files map[string]string) error {
+	if len(files) == 0 {
+		return nil
 	}
-	return
-}
 
-// ChannelByName returns the channel with the given name, or nil if not found
-func (m *Manifest) ChannelByName(name string) *ChannelBlock {
-	for _, c := range m.Channels {
-		if c.Name == name {
-			return &c
+	for k, v := range files {
+		f, err := manifest.FS.Open(k)
+		if err != nil {
+			return errors.WithStack(err)
 		}
+		err = f.Close()
+		if err != nil {
+			return errors.WithStack(err)
+		}
+		pkg.Files = append(pkg.Files, &ResolvedFileRef{
+			FromPath: k,
+			FS:       manifest.FS,
+			ToPAth:   v,
+		})
 	}
 	return nil
 }
+
+// mustAbs ensures that "path" is either empty or an absolute file path, after expansion.
+func mustAbs(action Action, path string) error {
+	if path == "" || filepath.IsAbs(path) {
+		return nil
+	}
+	return participle.Errorf(action.position(), "%q must be an absolute path", path)
+}
