@@ -2,17 +2,17 @@ package shell
 
 import (
 	"io"
-	"os"
 	"path/filepath"
 
-	"github.com/pkg/errors"
+	"github.com/cashapp/hermit/errors"
+	"github.com/cashapp/hermit/internal/system"
 )
 
-const bashShellHooks = commonHooks + `
+var bashShellHooks = `
 if test -n "${PROMPT_COMMAND+_}"; then 
-  export PROMPT_COMMAND="change_hermit_env; $PROMPT_COMMAND"
+  PROMPT_COMMAND="change_hermit_env; $PROMPT_COMMAND"
 else
-  export PROMPT_COMMAND="change_hermit_env"
+  PROMPT_COMMAND="change_hermit_env"
 fi
 
 complete -o nospace -C "$HOME/bin/hermit" hermit
@@ -35,7 +35,7 @@ func (sh *Bash) ActivationScript(w io.Writer, config ActivationConfig) error { /
 }
 
 func (sh *Bash) ActivationHooksInstallation() (path, script string, err error) { // nolint: golint
-	home, err := os.UserHomeDir()
+	home, err := system.UserHomeDir()
 	if err != nil {
 		return "", "", errors.WithStack(err)
 	}
@@ -44,5 +44,5 @@ func (sh *Bash) ActivationHooksInstallation() (path, script string, err error) {
 }
 
 func (sh *Bash) ActivationHooksCode() (script string, err error) { // nolint: golint
-	return bashShellHooks, err
+	return commonHooks + bashShellHooks, nil
 }
